@@ -1,4 +1,4 @@
 #! /usr/bin/env bash
-$EXTRACTRC *.ui >> rc.cpp
+$EXTRACTRC `find . -name "*.ui"` >> rc.cpp
 $XGETTEXT `find . -name "*.cpp" -o -name "*.qml"` -o $podir/kcm_colors.pot
 rm -f rc.cpp
