@@ -3,6 +3,7 @@
  * Copyright (C) 2007 Jeremy Whiting <jpwhiting@kde.org>
  * Copyright (C) 2016 Olivier Churlaud <olivier@churlaud.com>
  * Copyright (C) 2019 Kai Uwe Broulik <kde@privat.broulik.de>
+ * Copyright (c) 2019 Cyril Rossi <cyril.rossi@enioka.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -46,38 +47,47 @@
 #include <KIO/DeleteJob>
 #include <KIO/JobUiDelegate>
 
+#include <knewstuffcore_version.h>
+#if KNEWSTUFFCORE_VERSION_MAJOR==5 && KNEWSTUFFCORE_VERSION_MINOR>=68
+#include <KNSCore/EntryWrapper>
+#endif
+
 #include <algorithm>
 
 #include "../krdb/krdb.h"
 
 #include "colorsmodel.h"
 #include "filterproxymodel.h"
-
-static const QString s_defaultColorSchemeName = QStringLiteral("Breeze");
+#include "colorssettings.h"
 
 K_PLUGIN_FACTORY_WITH_JSON(KCMColorsFactory, "kcm_colors.json", registerPlugin<KCMColors>();)
 
 KCMColors::KCMColors(QObject *parent, const QVariantList &args)
-    : KQuickAddons::ConfigModule(parent, args)
+    : KQuickAddons::ManagedConfigModule(parent, args)
     , m_model(new ColorsModel(this))
     , m_filteredModel(new FilterProxyModel(this))
+    , m_settings(new ColorsSettings(this))
     , m_config(KSharedConfig::openConfig(QStringLiteral("kdeglobals")))
 {
     qmlRegisterUncreatableType<KCMColors>("org.kde.private.kcms.colors", 1, 0, "KCM", QStringLiteral("Cannot create instances of KCM"));
     qmlRegisterType<ColorsModel>();
     qmlRegisterType<FilterProxyModel>();
+    qmlRegisterType<ColorsSettings>();
 
     KAboutData *about = new KAboutData(QStringLiteral("kcm_colors"), i18n("Colors"),
                                        QStringLiteral("2.0"), QString(), KAboutLicense::GPL);
     about->addAuthor(i18n("Kai Uwe Broulik"), QString(), QStringLiteral("kde@privat.broulik.de"));
     setAboutData(about);
 
-    connect(m_model, &ColorsModel::selectedSchemeChanged, this, [this] {
+    connect(m_model, &ColorsModel::pendingDeletionsChanged, this, &KCMColors::settingsChanged);
+
+    connect(m_model, &ColorsModel::selectedSchemeChanged, this, [this](const QString &scheme) {
         m_selectedSchemeDirty = true;
-        setNeedsSave(true);
+        m_settings->setColorScheme(scheme);
     });
-    connect(m_model, &ColorsModel::pendingDeletionsChanged, this, [this] {
-        setNeedsSave(true);
+
+    connect(m_settings, &ColorsSettings::colorSchemeChanged, this, [this] {
+        m_model->setSelectedScheme(m_settings->colorScheme());
     });
 
     connect(m_model, &ColorsModel::selectedSchemeChanged, m_filteredModel, &FilterProxyModel::setSelectedScheme);
@@ -99,30 +109,31 @@ FilterProxyModel *KCMColors::filteredModel() const
     return m_filteredModel;
 }
 
+ColorsSettings *KCMColors::colorsSettings() const
+{
+    return m_settings;
+}
+
 bool KCMColors::downloadingFile() const
 {
     return m_tempCopyJob;
 }
 
-void KCMColors::getNewStuff(QQuickItem *ctx)
+void KCMColors::reloadModel(const QQmlListReference &changedEntries)
 {
-    if (!m_newStuffDialog) {
-        m_newStuffDialog = new KNS3::DownloadDialog(QStringLiteral("colorschemes.knsrc"));
-        m_newStuffDialog.data()->setWindowTitle(i18n("Download New Color Schemes"));
-        m_newStuffDialog->setWindowModality(Qt::WindowModal);
-        m_newStuffDialog->winId(); // so it creates the windowHandle();
+    m_model->load();
 
-        connect(m_newStuffDialog.data(), &KNS3::DownloadDialog::accepted, this, [this] {
-            m_model->load();
+#if KNEWSTUFFCORE_VERSION_MAJOR==5 && KNEWSTUFFCORE_VERSION_MINOR>=68
+    // If a new theme was installed, select the first color file in it
+    if (changedEntries.count() > 0) {
+        QStringList installedThemes;
 
-            const auto newEntries = m_newStuffDialog->installedEntries();
-            // If one new theme was installed, select the first color file in it
-            if (newEntries.count() == 1) {
-                QStringList installedThemes;
+        const QString suffix = QStringLiteral(".colors");
 
-                const QString suffix = QStringLiteral(".colors");
-
-                for (const QString &path : newEntries.first().installedFiles()) {
+        for (int i = 0; i < changedEntries.count(); ++i) {
+            KNSCore::EntryWrapper* entry = qobject_cast<KNSCore::EntryWrapper*>(changedEntries.at(i));
+            if (entry && entry->entry().status() == KNS3::Entry::Installed) {
+                for (const QString &path : entry->entry().installedFiles()) {
                     const QString fileName = path.section(QLatin1Char('/'), -1, -1);
 
                     const int suffixPos = fileName.indexOf(suffix);
@@ -140,15 +151,12 @@ void KCMColors::getNewStuff(QQuickItem *ctx)
 
                     m_model->setSelectedScheme(installedThemes.constFirst());
                 }
+                // Only do this for the first newly installed theme we find
+                break;
             }
-        });
+        }
     }
-
-    if (ctx && ctx->window()) {
-        m_newStuffDialog->windowHandle()->setTransientParent(ctx->window());
-    }
-
-    m_newStuffDialog.data()->show();
+#endif
 }
 
 void KCMColors::installSchemeFromFile(const QUrl &url)
@@ -291,22 +299,28 @@ void KCMColors::editScheme(const QString &schemeName, QQuickItem *ctx)
     m_editDialogProcess->start(QStringLiteral("kcolorschemeeditor"), args);
 }
 
+bool KCMColors::isSaveNeeded() const
+{
+    return !m_model->match(m_model->index(0, 0), ColorsModel::PendingDeletionRole, true).isEmpty();
+}
+
+
 void KCMColors::load()
 {
+    ManagedConfigModule::load();
     m_model->load();
 
     m_config->markAsClean();
     m_config->reparseConfiguration();
 
-    KConfigGroup group(m_config, "General");
-    const QString schemeName = group.readEntry("ColorScheme", s_defaultColorSchemeName);
+    const QString schemeName = m_settings->colorScheme();
 
     // If the scheme named in kdeglobals doesn't exist, show a warning and use default scheme
     if (m_model->indexOfScheme(schemeName) == -1) {
-        m_model->setSelectedScheme(s_defaultColorSchemeName);
+        m_model->setSelectedScheme(m_settings->defaultColorSchemeValue());
         // These are normally synced but initially the model doesn't emit a change to avoid the
         // Apply button from being enabled without any user interaction. Sync manually here.
-        m_filteredModel->setSelectedScheme(s_defaultColorSchemeName);
+        m_filteredModel->setSelectedScheme(m_settings->defaultColorSchemeValue());
         emit showSchemeNotInstalledWarning(schemeName);
     } else {
         m_model->setSelectedScheme(schemeName);
@@ -315,6 +329,7 @@ void KCMColors::load()
 
     {
         KConfig cfg(QStringLiteral("kcmdisplayrc"), KConfig::NoGlobals);
+        KConfigGroup group(m_config, "General");
         group = KConfigGroup(&cfg, "X11");
         m_applyToAlien = group.readEntry("exportKDEColors", true);
     }
@@ -322,20 +337,16 @@ void KCMColors::load()
 
 void KCMColors::save()
 {
+    ManagedConfigModule::save();
     if (m_selectedSchemeDirty) {
         saveColors();
     }
 
     processPendingDeletions();
-
-    setNeedsSave(false);
 }
 
 void KCMColors::saveColors()
 {
-    KConfigGroup grp(m_config, "General");
-    grp.writeEntry("ColorScheme", m_model->selectedScheme());
-
     const QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
         QStringLiteral("color-schemes/%1.colors").arg(m_model->selectedScheme()));
 
@@ -464,11 +475,4 @@ void KCMColors::processPendingDeletions()
     m_model->removeItemsPendingDeletion();
 }
 
-void KCMColors::defaults()
-{
-    m_model->setSelectedScheme(s_defaultColorSchemeName);
-
-    setNeedsSave(true);
-}
-
 #include "colors.moc"
