@@ -4,6 +4,7 @@
     Copyright 2000 Rik Hemsley
     Copyright 2015 Antonis Tsiapaliokas <antonis.tsiapaliokas@kde.org>
     Copyright 2017 Marco Martin <mart@kde.org>
+    Copyright 2019 Benjamin Port <benjamin.port@enioka.com>
 
     Ported to kcontrol2 by Geert Jansen.
 
@@ -44,11 +45,12 @@
 #include <KPluginFactory>
 #include <KFontDialog>
 #include <KWindowSystem>
-#include <KMessageBox>
 
 #include "../krdb/krdb.h"
 #include "previewimageprovider.h"
 
+#include "fontssettings.h"
+
 /**** DLL Interface ****/
 K_PLUGIN_FACTORY_WITH_JSON(KFontsFactory, "kcm_fonts.json", registerPlugin<KFonts>();)
 
@@ -147,23 +149,28 @@ void FontAASettings::load()
     excludeToChanged();
     excludeFromChanged();
 
-    KXftConfig::SubPixel::Type spType;
-    if (!xft.getSubPixelType(spType) || KXftConfig::SubPixel::NotSet == spType) {
+    // start with empty subpixel type
+    KXftConfig::SubPixel::Type spType = KXftConfig::SubPixel::NotSet;
+    // get subpixel type from config
+    xft.getSubPixelType(spType);
+    m_originalState.subPixel = spType;
+    // if it is not set, we set it to rgb
+    if (spType == KXftConfig::SubPixel::NotSet) {
         spType = KXftConfig::SubPixel::Rgb;
     }
-
     setSubPixel(spType);
-    m_originalState.subPixel = spType;
     m_state.subPixelHasLocalConfig = xft.subPixelTypeHasLocalConfig();
 
-    KXftConfig::Hint::Style hStyle;
-
-    if (!xft.getHintStyle(hStyle) || KXftConfig::Hint::NotSet == hStyle) {
+    // start with empty hint style
+    KXftConfig::Hint::Style hStyle = KXftConfig::Hint::NotSet;
+    // get value from config;
+    xft.getHintStyle(hStyle);
+    m_originalState.hinting = hStyle;
+    // if it is not set, we set it to slight hinting
+    if (hStyle == KXftConfig::Hint::NotSet) {
         hStyle = KXftConfig::Hint::Slight;
     }
-
     setHinting(hStyle);
-    m_originalState.hinting = hStyle;
     m_state.hintingHasLocalConfig = xft.hintStyleHasLocalConfig();
 
     KConfig _cfgfonts("kcmfonts");
@@ -313,6 +320,7 @@ bool FontAASettings::save(KXftConfig::AntiAliasing::State aaState)
 
 void FontAASettings::defaults()
 {
+    setExclude(false);
     setExcludeTo(15);
     setExcludeFrom(8);
     setAntiAliasing(true);
@@ -342,7 +350,7 @@ bool FontAASettings::exclude() const
     return m_state.exclude;
 }
 
-void FontAASettings::setExcludeTo(const int &excludeTo)
+void FontAASettings::setExcludeTo(int excludeTo)
 {
     if (m_state.excludeTo == excludeTo) {
         return;
@@ -357,14 +365,14 @@ int FontAASettings::excludeTo() const
     return m_state.excludeTo;
 }
 
-void FontAASettings::setExcludeFrom(const int &excludeTo)
+void FontAASettings::setExcludeFrom(int excludeFrom)
 {
-    if (m_state.excludeFrom == excludeTo) {
+    if (m_state.excludeFrom == excludeFrom) {
         return;
     }
 
-    m_state.excludeFrom = excludeTo;
-    emit excludeToChanged();
+    m_state.excludeFrom = excludeFrom;
+    emit excludeFromChanged();
 }
 
 int FontAASettings::excludeFrom() const
@@ -462,288 +470,6 @@ bool FontAASettings::needsSave() const
     return m_state != m_originalState;
 }
 
-
-/**** KFonts ****/
-
-KFonts::KFonts(QObject *parent, const QVariantList &args)
-    : KQuickAddons::ConfigModule(parent, args)
-    , m_fontAASettings(new FontAASettings(this))
-{
-    KAboutData* about = new KAboutData("kcm_fonts", i18n("Fonts"),
-                                       "0.1", QString(), KAboutLicense::LGPL);
-    about->addAuthor(i18n("Antonis Tsiapaliokas"), QString(), "antonis.tsiapaliokas@kde.org");
-    setAboutData(about);
-    qmlRegisterType<QStandardItemModel>();
-    setButtons(Apply | Default | Help);
-
-    auto updateState = [this]() {
-        setNeedsSave(m_fontAASettings->needsSave());
-    };
-
-    connect(m_fontAASettings, &FontAASettings::subPixelCurrentIndexChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::hintingCurrentIndexChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::excludeChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::excludeFromChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::excludeToChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::antiAliasingChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::aliasingChanged, this, updateState);
-    connect(m_fontAASettings, &FontAASettings::dpiChanged, this, updateState);
-}
-
-KFonts::~KFonts()
-{
-}
-
-void KFonts::defaults()
-{
-#ifdef Q_OS_MAC
-    setGeneralFont(QFont("Lucida Grande", 13));
-    setMenuFont(QFont("Lucida Grande", 13));
-    setFixedWidthFont(QFont("Monaco", 10));
-    setToolbarFont(QFont("Lucida Grande", 11));
-    setSmallFont(QFont("Lucida Grande", 9));
-    setWindowTitleFont(QFont("Lucida Grande", 14));
-#else
-    setGeneralFont(QFont("Noto Sans", 10));
-    setMenuFont(QFont("Noto Sans", 10));
-    setFixedWidthFont(QFont("Hack", 9));
-    setToolbarFont(QFont("Noto Sans", 10));
-    setSmallFont(QFont("Noto Sans", 8));
-    setWindowTitleFont(QFont("Noto Sans", 10));
-#endif
-
-    m_fontAASettings->defaults();
-}
-
-void KFonts::load()
-{
-    KSharedConfig::Ptr config = KSharedConfig::openConfig("kdeglobals");
-
-    KConfigGroup cg(config, "General");
-    m_generalFont = m_generalFontOriginal = nearestExistingFont(cg.readEntry("font", m_defaultFont));
-
-
-    m_fixedWidthFont = m_fixedWidthFontOriginal = nearestExistingFont(cg.readEntry("fixed", QFont("Hack", 9)));
-
-    m_smallFont = m_smallFontOriginal = nearestExistingFont(cg.readEntry("smallestReadableFont", m_defaultFont));
-
-    m_toolbarFont = m_toolbarFontOriginal = nearestExistingFont(cg.readEntry("toolBarFont", m_defaultFont));
-
-    m_menuFont = m_menuFontOriginal = nearestExistingFont(cg.readEntry("menuFont", m_defaultFont));
-
-    cg = KConfigGroup(config, "WM");
-    m_windowTitleFont = m_windowTitleFontOriginal = nearestExistingFont(cg.readEntry("activeFont", m_defaultFont));
-
-    engine()->addImageProvider("preview", new PreviewImageProvider(generalFont()));
-
-    emit generalFontChanged();
-    emit fixedWidthFontChanged();
-    emit smallFontChanged();
-    emit toolbarFontChanged();
-    emit menuFontChanged();
-    emit windowTitleFontChanged();
-
-    m_fontAASettings->load();
-    setNeedsSave(false);
-}
-
-void KFonts::save()
-{
-    KSharedConfig::Ptr config = KSharedConfig::openConfig("kdeglobals");
-
-    KConfigGroup cg(config, "General");
-    cg.writeEntry("font", m_generalFont.toString());
-    cg.writeEntry("fixed", m_fixedWidthFont.toString());
-    cg.writeEntry("smallestReadableFont", m_smallFont.toString());
-    cg.writeEntry("toolBarFont", m_toolbarFont.toString());
-    cg.writeEntry("menuFont", m_menuFont.toString());
-    cg.sync();
-    cg = KConfigGroup(config, "WM");
-    cg.writeEntry("activeFont", m_windowTitleFont.toString());
-    cg.sync();
-
-    m_defaultFontOriginal = m_defaultFont;
-    m_generalFontOriginal = m_generalFont;
-    m_fixedWidthFontOriginal = m_fixedWidthFont;
-    m_smallFontOriginal = m_smallFont;
-    m_toolbarFontOriginal = m_toolbarFont;
-    m_menuFontOriginal = m_menuFont;
-    m_windowTitleFontOriginal = m_windowTitleFont;
-
-    KConfig _cfgfonts("kcmfonts");
-    KConfigGroup cfgfonts(&_cfgfonts, "General");
-
-    FontAASettings::AASetting aaSetting = FontAASettings::AASystem;
-    if (m_fontAASettings->antiAliasingNeedsSave()) {
-        aaSetting = m_fontAASettings->antiAliasing() ? FontAASettings::AAEnabled : FontAASettings::AADisabled;
-    }
-    cfgfonts.writeEntry("dontChangeAASettings", aaSetting == FontAASettings::AASystem);
-
-    if (aaSetting == FontAASettings::AAEnabled) {
-        m_fontAASettings->save(KXftConfig::AntiAliasing::Enabled);
-    } else if (aaSetting == FontAASettings::AADisabled) {
-        m_fontAASettings->save(KXftConfig::AntiAliasing::Disabled);
-    } else {
-        m_fontAASettings->save(KXftConfig::AntiAliasing::NotSet);
-    }
-
-    KGlobalSettings::self()->emitChange(KGlobalSettings::FontChanged);
-
-    runRdb(KRdbExportXftSettings | KRdbExportGtkTheme);
-
-    emit fontsHaveChanged();
-    setNeedsSave(false);
-}
-
-void KFonts::updateNeedsSave()
-{
-    setNeedsSave(m_defaultFontOriginal != m_defaultFont ||
-                 m_generalFontOriginal != m_generalFont ||
-                 m_fixedWidthFontOriginal != m_fixedWidthFont ||
-                 m_smallFontOriginal != m_smallFont ||
-                 m_toolbarFontOriginal != m_toolbarFont ||
-                 m_menuFontOriginal != m_menuFont ||
-                 m_windowTitleFontOriginal != m_windowTitleFont ||
-                m_fontAASettings->needsSave());
-}
-
-void KFonts::setGeneralFont(const QFont &font)
-{
-    if (m_generalFont == font) {
-        return;
-    }
-
-    m_generalFont = font;
-    emit generalFontChanged();
-    updateNeedsSave();
-}
-
-QFont KFonts::generalFont() const
-{
-    return m_generalFont;
-}
-
-void KFonts::setFixedWidthFont(const QFont &font)
-{
-    if (m_fixedWidthFont == font) {
-        return;
-    }
-
-    m_fixedWidthFont = font;
-    emit fixedWidthFontChanged();
-    updateNeedsSave();
-}
-
-QFont KFonts::fixedWidthFont() const
-{
-    return m_fixedWidthFont;
-}
-
-void KFonts::setSmallFont(const QFont &font)
-{
-    if (m_smallFont == font) {
-        return;
-    }
-
-    m_smallFont = font;
-    emit smallFontChanged();
-    updateNeedsSave();
-}
-
-QFont KFonts::smallFont() const
-{
-    return m_smallFont;
-}
-
-void KFonts::setToolbarFont(const QFont &font)
-{
-    if (m_toolbarFont == font) {
-        return;
-    }
-
-    m_toolbarFont = font;
-    emit toolbarFontChanged();
-    updateNeedsSave();
-}
-
-QFont KFonts::toolbarFont() const
-{
-    return m_toolbarFont;
-}
-
-void KFonts::setMenuFont(const QFont &font)
-{
-    if (m_menuFont == font) {
-        return;
-    }
-
-    m_menuFont = font;
-    emit menuFontChanged();
-    updateNeedsSave();
-}
-
-QFont KFonts::menuFont() const
-{
-    return m_menuFont;
-}
-
-void KFonts::setWindowTitleFont(const QFont &font)
-{
-    if (m_windowTitleFont == font) {
-        return;
-    }
-
-    m_windowTitleFont = font;
-    emit windowTitleFontChanged();
-    updateNeedsSave();
-}
-
-QFont KFonts::windowTitleFont() const
-{
-    return m_windowTitleFont;
-}
-
-void KFonts::adjustAllFonts()
-{
-    QFont font = m_generalFont;
-    KFontChooser::FontDiffFlags fontDiffFlags;
-    int ret = KFontDialog::getFontDiff(font, fontDiffFlags, KFontChooser::NoDisplayFlags);
-
-    if (ret == KDialog::Accepted && fontDiffFlags) {
-        setGeneralFont(applyFontDiff(m_generalFont, font, fontDiffFlags));
-        setMenuFont(applyFontDiff(m_menuFont, font, fontDiffFlags));
-        {
-            const QFont adjustedFont = applyFontDiff(m_fixedWidthFont, font, fontDiffFlags);
-            if (QFontInfo(adjustedFont).fixedPitch()) {
-                setFixedWidthFont(adjustedFont);
-            }
-        }
-        setToolbarFont(applyFontDiff(m_toolbarFont, font, fontDiffFlags));
-        setSmallFont(applyFontDiff(m_smallFont, font, fontDiffFlags));
-        setWindowTitleFont(applyFontDiff(m_windowTitleFont, font, fontDiffFlags));
-    }
-}
-
-QFont KFonts::applyFontDiff(const QFont &fnt, const QFont &newFont, int fontDiffFlags)
-{
-    QFont font(fnt);
-
-    if (fontDiffFlags & KFontChooser::FontDiffSize) {
-        font.setPointSizeF(newFont.pointSizeF());
-    }
-    if ((fontDiffFlags & KFontChooser::FontDiffFamily)) {
-        font.setFamily(newFont.family());
-    }
-    if (fontDiffFlags & KFontChooser::FontDiffStyle) {
-        font.setWeight(newFont.weight());
-        font.setStyle(newFont.style());
-        font.setUnderline(newFont.underline());
-        font.setStyleName(newFont.styleName());
-    }
-
-    return font;
-}
-
 bool FontAASettings::State::operator==(const State& other) const
 {
     if (
@@ -768,5 +494,161 @@ bool FontAASettings::State::operator!=(const State& other) const
     return !(*this == other);
 }
 
+/**** KFonts ****/
+
+KFonts::KFonts(QObject *parent, const QVariantList &args)
+    : KQuickAddons::ManagedConfigModule(parent, args)
+    , m_settings(new FontsSettings(this))
+    , m_fontAASettings(new FontAASettings(this))
+{
+    KAboutData* about = new KAboutData("kcm_fonts", i18n("Fonts"),
+                                       "0.1", QString(), KAboutLicense::LGPL);
+    about->addAuthor(i18n("Antonis Tsiapaliokas"), QString(), "antonis.tsiapaliokas@kde.org");
+    setAboutData(about);
+    qmlRegisterType<QStandardItemModel>();
+    qmlRegisterType<FontsSettings>();
+
+    setButtons(Apply | Default | Help);
+
+    auto updateState = [this]() {
+        setNeedsSave(m_fontAASettings->needsSave());
+    };
+
+    connect(m_fontAASettings, &FontAASettings::subPixelCurrentIndexChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::hintingCurrentIndexChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::excludeChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::excludeFromChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::excludeToChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::antiAliasingChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::aliasingChanged, this, updateState);
+    connect(m_fontAASettings, &FontAASettings::dpiChanged, this, updateState);
+}
+
+KFonts::~KFonts()
+{
+}
+
+FontsSettings *KFonts::fontsSettings() const
+{
+    return m_settings;
+}
+
+void KFonts::defaults()
+{
+    ManagedConfigModule::defaults();
+    m_fontAASettings->defaults();
+}
+
+void KFonts::setNearestExistingFonts()
+{
+    m_settings->setFont(nearestExistingFont(m_settings->font()));
+    m_settings->setFixed(nearestExistingFont(m_settings->fixed()));
+    m_settings->setSmallestReadableFont(nearestExistingFont(m_settings->smallestReadableFont()));
+    m_settings->setToolBarFont(nearestExistingFont(m_settings->toolBarFont()));
+    m_settings->setMenuFont(nearestExistingFont(m_settings->menuFont()));
+    m_settings->setActiveFont(nearestExistingFont(m_settings->activeFont()));
+}
+
+void KFonts::load()
+{
+    // first load all the settings
+    ManagedConfigModule::load();
+    m_fontAASettings->load();
+
+    // Then set the existing fonts based on those settings
+    setNearestExistingFonts();
+
+    // Load preview
+    // NOTE: This needs to be done AFTER AA settings is loaded
+    // otherwise AA settings will be resetted in process of loading
+    // previews
+    engine()->addImageProvider("preview", new PreviewImageProvider(m_settings->font()));
+
+    // reload state after loading by emiting a settings signal
+    emit m_settings->activeFontChanged();
+}
+
+void KFonts::save()
+{
+    ManagedConfigModule::save();
+
+    KConfig _cfgfonts("kcmfonts");
+    KConfigGroup cfgfonts(&_cfgfonts, "General");
+
+    FontAASettings::AASetting aaSetting = FontAASettings::AASystem;
+    if (m_fontAASettings->antiAliasingNeedsSave()) {
+        aaSetting = m_fontAASettings->antiAliasing() ? FontAASettings::AAEnabled : FontAASettings::AADisabled;
+    }
+    cfgfonts.writeEntry("dontChangeAASettings", aaSetting == FontAASettings::AASystem);
+
+    if (aaSetting == FontAASettings::AAEnabled) {
+        m_fontAASettings->save(KXftConfig::AntiAliasing::Enabled);
+    } else if (aaSetting == FontAASettings::AADisabled) {
+        m_fontAASettings->save(KXftConfig::AntiAliasing::Disabled);
+    } else {
+        m_fontAASettings->save(KXftConfig::AntiAliasing::NotSet);
+    }
+
+    KGlobalSettings::self()->emitChange(KGlobalSettings::FontChanged);
+
+    runRdb(KRdbExportXftSettings | KRdbExportGtkTheme);
+
+    emit fontsHaveChanged();
+}
+
+bool KFonts::isSaveNeeded() const
+{
+    return m_fontAASettings->needsSave();
+}
+
+void KFonts::adjustAllFonts()
+{
+    QFont font = m_settings->font();
+    KFontChooser::FontDiffFlags fontDiffFlags;
+    int ret = KFontDialog::getFontDiff(font, fontDiffFlags, KFontChooser::NoDisplayFlags);
+
+    if (ret == KDialog::Accepted && fontDiffFlags) {
+        if (!m_settings->isImmutable("font")) {
+            m_settings->setFont(applyFontDiff(m_settings->font(), font, fontDiffFlags));
+        }
+        if (!m_settings->isImmutable("menuFont")) {
+            m_settings->setMenuFont(applyFontDiff(m_settings->menuFont(), font, fontDiffFlags));
+        }
+        if (!m_settings->isImmutable("toolBarFont")) {
+            m_settings->setToolBarFont(applyFontDiff(m_settings->toolBarFont(), font, fontDiffFlags));
+        }
+        if (!m_settings->isImmutable("activeFont")) {
+            m_settings->setActiveFont(applyFontDiff(m_settings->activeFont(), font, fontDiffFlags));
+        }
+        if (!m_settings->isImmutable("smallestReadableFont")) {
+            m_settings->setSmallestReadableFont(applyFontDiff(m_settings->smallestReadableFont(), font, fontDiffFlags));
+        }
+        const QFont adjustedFont = applyFontDiff(m_settings->fixed(), font, fontDiffFlags);
+        if (QFontInfo(adjustedFont).fixedPitch() && !m_settings->isImmutable("fixed")) {
+            m_settings->setFixed(adjustedFont);
+        }
+    }
+}
+
+QFont KFonts::applyFontDiff(const QFont &fnt, const QFont &newFont, int fontDiffFlags)
+{
+    QFont font(fnt);
+
+    if (fontDiffFlags & KFontChooser::FontDiffSize) {
+        font.setPointSizeF(newFont.pointSizeF());
+    }
+    if ((fontDiffFlags & KFontChooser::FontDiffFamily)) {
+        font.setFamily(newFont.family());
+    }
+    if (fontDiffFlags & KFontChooser::FontDiffStyle) {
+        font.setWeight(newFont.weight());
+        font.setStyle(newFont.style());
+        font.setUnderline(newFont.underline());
+        font.setStyleName(newFont.styleName());
+    }
+
+    return font;
+}
+
 #include "fonts.moc"
 
